<?php
/**
 * Created by PhpStorm.
 * User: steve
 * Date: 23/08/18
 * Time: 14:35
 */

namespace Drupal\event_scheduler;

use Drupal\event_scheduler\Event\EventScheduleInterface;

/**
 * Trait EventSchedulerTrait
 *
 * @package Drupal\event_scheduler
 */
trait EventSchedulerTrait {

  protected int $id = 0;

  protected string $name;

  protected string $class;

  protected string $tag;

  protected int $launch;

  protected bool $processed;

  public function initialise(int $launch, string $tag = ''): EventScheduleInterface {
    return $this
      ->setLaunch($launch)
      ->setTag($tag)
      ->setName(static::NAME)
      ->setClass(get_class($this));
  }

  /**
   * @param int $id
   *
   * @return $this|EventScheduleInterface
   */
  public function setId(int $id): EventScheduleInterface {
    $this->id = $id;
    return $this;
  }

  /**
   * @return int
   */
  public function id(): int {
    return $this->id;
  }

  /**
   * @param string $name
   *
   * @return $this|EventScheduleInterface
   */
  public function setName(string $name): EventScheduleInterface {
    $this->name = $name;
    return $this;
  }

  /**
   * @return string
   */
  public function getName(): string {
    return $this->name;
  }

  /**
   * @param string $event_class
   *
   * @return $this|EventScheduleInterface
   */
  public function setClass(string $event_class): EventScheduleInterface {
    $this->class = $event_class;
    return $this;
  }

  /**
   * @return string
   */
  public function getClass(): string {
    return $this->class;
  }

  /**
   * @param string $tag
   *
   * @return $this|EventScheduleInterface
   */
  public function setTag(string $tag): EventScheduleInterface {
    $this->tag = $tag;
    return $this;
  }

  /**
   * @return string
   */
  public function getTag(): string {
    return $this->tag;
  }

  /**
   * @param int $when UTC timestamp
   *
   * @return $this|EventScheduleInterface
   */
  public function setLaunch(int $when): EventScheduleInterface {
    $this->launch = $when;
    return $this;
  }

  /**
   * @return int UTC timestamp
   */
  public function getLaunch(): int {
    return $this->launch;
  }

  /**
   * @param bool
   *
   * @return $this|EventScheduleInterface
   */
  public function setProcessed($processed = TRUE): EventScheduleInterface {
    $this->processed = $processed;
    return $this;
  }

  /**
   * @return bool
   */
  public function getProcessed(): bool {
    return $this->processed;
  }

}
